'use strict';

exports.__esModule = true;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

exports.toInstallOptions = toInstallOptions;
exports.install = install;

var _Project = require('../Project');

var _Project2 = _interopRequireDefault(_Project);

var _options = require('../utils/options');

var options = _interopRequireWildcard(_options);

var _processes = require('../utils/processes');

var processes = _interopRequireWildcard(_processes);

var _fs = require('../utils/fs');

var fs = _interopRequireWildcard(_fs);

var _path = require('path');

var path = _interopRequireWildcard(_path);

var _logger = require('../utils/logger');

var logger = _interopRequireWildcard(_logger);

var _messages = require('../utils/messages');

var messages = _interopRequireWildcard(_messages);

var _yarn = require('../utils/yarn');

var yarn = _interopRequireWildcard(_yarn);

var _pathIsInside = require('path-is-inside');

var _pathIsInside2 = _interopRequireDefault(_pathIsInside);

var _errors = require('../utils/errors');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function toInstallOptions(args, flags) {
  return {
    cwd: options.string(flags.cwd, 'cwd')
  };
}

async function install(opts) {
  let cwd = opts.cwd || process.cwd();
  let project = await _Project2.default.init(cwd);
  let workspaces = await project.getWorkspaces();

  let {
    graph: dependencyGraph,
    valid: dependencyGraphValid
  } = await project.getDependencyGraph(workspaces);

  let projectDependencies = project.pkg.getAllDependencies();

  let directoriesToCreate = [];
  let symlinksToCreate = [];
  let valid = true;

  let workspacesToDependencies = {};

  for (let workspace of workspaces) {
    let dependencies = workspace.pkg.getAllDependencies();

    workspacesToDependencies[workspace.pkg.config.getName()] = dependencies;

    directoriesToCreate.push(workspace.pkg.nodeModules);
    directoriesToCreate.push(workspace.pkg.nodeModulesBin);

    for (let [name, version] of dependencies) {
      let matched = projectDependencies.get(name);

      if (dependencyGraph.has(name)) {
        continue;
      }

      if (!matched) {
        valid = false;
        logger.error(messages.depMustBeAddedToProject(workspace.pkg.config.getName(), name));
        continue;
      }

      if (version !== matched) {
        valid = false;
        logger.error(messages.depMustMatchProject(workspace.pkg.config.getName(), name, matched, version));
        continue;
      }

      let src = path.join(project.pkg.nodeModules, name);
      let dest = path.join(workspace.pkg.nodeModules, name);

      symlinksToCreate.push({ src, dest, type: 'junction' });
    }
  }

  for (let [name, node] of dependencyGraph) {
    let nodeModules = path.join(node.pkg.dir, 'node_modules');

    for (let dependency of node.dependencies) {
      let depWorkspace = dependencyGraph.get(dependency);

      if (!depWorkspace) {
        throw new _errors.BoltError(`Missing workspace: "${dependency}"`);
      }

      let src = depWorkspace.pkg.dir;
      let dest = path.join(nodeModules, dependency);

      symlinksToCreate.push({ src, dest, type: 'junction' });
    }
  }

  if (!dependencyGraphValid || !valid) {
    throw new _errors.BoltError('Cannot symlink invalid set of dependencies.');
  }

  await _Project2.default.runWorkspaceTasks(workspaces, async workspace => {
    await yarn.run(workspace.pkg, 'preinstall');
  });

  logger.log('[1/2] Installing project dependencies...');

  await processes.spawn('yarn', ['install', '--non-interactive', '-s'], {
    cwd: project.pkg.dir
  });

  logger.log('[2/2] Linking workspace dependencies...');

  for (let binFile of await fs.readdirSafe(project.pkg.nodeModulesBin)) {
    let binPath = path.join(project.pkg.nodeModulesBin, binFile);
    let binName = path.basename(binPath);

    let linkFile = await fs.readlink(binPath);

    if (!linkFile) {
      throw new _errors.BoltError(`${binName} is not a symlink`);
    }

    let linkPath = path.join(project.pkg.nodeModulesBin, linkFile);

    if (!(0, _pathIsInside2.default)(linkPath, project.pkg.nodeModules)) {
      throw new _errors.BoltError(`${binName} is linked to a location outside of project node_modules: ${linkPath}`);
    }

    let relativeLinkPath = path.relative(project.pkg.nodeModules, linkPath);
    let pathParts = relativeLinkPath.split(path.sep);
    let pkgName = pathParts[0];

    if (pkgName.startsWith('@')) {
      pkgName += '/' + pathParts[1];
    }

    for (let workspace of workspaces) {
      let dependencies = workspacesToDependencies[workspace.pkg.config.getName()];

      if (!dependencies.has(pkgName)) {
        continue;
      }

      let workspaceBinPath = path.join(workspace.pkg.nodeModulesBin, binName);

      symlinksToCreate.push({
        src: binPath,
        dest: workspaceBinPath,
        type: 'exec'
      });
    }
  }

  await _promise2.default.all(directoriesToCreate.map(dirName => {
    return fs.mkdirp(dirName);
  }));

  await _promise2.default.all(symlinksToCreate.map(async ({ src, dest, type }) => {
    await fs.symlink(src, dest, type);
  }));

  await _Project2.default.runWorkspaceTasks(workspaces, async workspace => {
    await yarn.run(workspace.pkg, 'postinstall');
    await yarn.run(workspace.pkg, 'prepublish');
  });

  logger.success('Installed and linked workspaces.');
}